@@ -19,14 +19,14 @@ class MultiqcModule(BaseMultiqcModule):
             name="bcl2fastq",
             anchor="bcl2fastq",
             href="https://support.illumina.com/sequencing/sequencing_software/bcl2fastq-conversion-software.html",
-            info="can be used to both demultiplex data and convert BCL files"
-            " to FASTQ file formats for downstream analysis.",
+            info="can be used to both demultiplex data and convert BCL files to FASTQ file formats for downstream analysis.",
+            # Can't find a DOI // doi=
         )
 
         # Gather data from all json files
         self.bcl2fastq_data = dict()
-        for myfile in self.find_log_files("bcl2fastq"):
-            self.parse_file_as_json(myfile)
+        for f in self.find_log_files("bcl2fastq"):
+            self.parse_file_as_json(f)
 
         # Collect counts by lane and sample (+source_files)
         self.bcl2fastq_bylane = dict()
@@ -102,7 +102,7 @@ class MultiqcModule(BaseMultiqcModule):
             anchor="bcl2fastq-bysample",
             description="Number of reads per sample.",
             helptext="""Perfect index reads are those that do not have a single mismatch.
-                All samples are aggregated across lanes combinned. Undetermined reads are ignored.
+                All samples are aggregated across lanes combined. Undetermined reads are ignored.
                 Undetermined reads are treated as a separate sample.""",
             plot=bargraph.plot(
                 [self.get_bar_data_from_counts(self.bcl2fastq_bysample), self.bcl2fastq_bysample_lane],
@@ -136,13 +136,16 @@ class MultiqcModule(BaseMultiqcModule):
             ),
         )
 
-    def parse_file_as_json(self, myfile):
+    def parse_file_as_json(self, f):
         try:
-            content = json.loads(myfile["f"])
+            content = json.loads(f["f"])
         except ValueError:
-            log.warning("Could not parse file as json: {}".format(myfile["fn"]))
+            log.warning("Could not parse file as json: {}".format(f["fn"]))
             return
-        runId = content["RunId"]
+
+        # Clean / prepend directories to sample names
+        runId = self.clean_s_name(content["RunId"], f)
+
         if runId not in self.bcl2fastq_data:
             self.bcl2fastq_data[runId] = dict()
         run_data = self.bcl2fastq_data[runId]
@@ -177,6 +180,7 @@ class MultiqcModule(BaseMultiqcModule):
                     sample = demuxResult["SampleName"]
                 else:
                     sample = "{}-{}".format(demuxResult["SampleId"], demuxResult["SampleName"])
+                sample = self.clean_s_name(sample, f)
                 if sample in run_data[lane]["samples"]:
                     log.debug(
                         "Duplicate runId/lane/sample combination found! Overwriting: {}, {}".format(
@@ -187,7 +191,7 @@ class MultiqcModule(BaseMultiqcModule):
                     "total": 0,
                     "total_yield": 0,
                     "perfectIndex": 0,
-                    "filename": os.path.join(myfile["root"], myfile["fn"]),
+                    "filename": os.path.join(f["root"], f["fn"]),
                     "yieldQ30": 0,
                     "qscore_sum": 0,
                 }
@@ -254,6 +258,7 @@ class MultiqcModule(BaseMultiqcModule):
                 lane["mean_qscore"] = float(lane["qscore_sum"]) / float(lane["total_yield"])
             except ZeroDivisionError:
                 lane["mean_qscore"] = "NA"
+
             for sample_id, sample in lane["samples"].items():
                 try:
                     sample["percent_Q30"] = (float(sample["yieldQ30"]) / float(sample["total_yield"])) * 100.0
@@ -395,7 +400,7 @@ class MultiqcModule(BaseMultiqcModule):
             "shared_key": "read_count",
         }
         headers["yieldQ30"] = {
-            "title": "{} Yield &ge; Q30".format(config.base_count_prefix),
+            "title": "Yield ({}) &ge; Q30".format(config.base_count_prefix),
             "description": "Number of bases with a Phred score of 30 or higher ({})".format(config.base_count_desc),
             "scale": "Greens",
             "shared_key": "base_count",
@@ -440,7 +445,7 @@ class MultiqcModule(BaseMultiqcModule):
         self.general_stats_addcols(data, headers)
 
     def lane_stats_table(self):
-        """ Return a table with overview stats for each bcl2fastq lane for a single flow cell """
+        """Return a table with overview stats for each bcl2fastq lane for a single flow cell"""
         headers = OrderedDict()
         headers["total_yield"] = {
             "title": "{} Total Yield".format(config.base_count_prefix),
