@@ -105,9 +105,7 @@ def custom_module_classes():
                     parsed_data["id"] = parsed_data.get("id", f["s_name"])
                     # Run sample-name cleaning on the data keys
                     try:
-                        parsed_data["data"] = {
-                            bm.clean_s_name(k, f["root"]): v for k, v in parsed_data.get("data", {}).items()
-                        }
+                        parsed_data["data"] = {bm.clean_s_name(k, f): v for k, v in parsed_data.get("data", {}).items()}
                     except AttributeError as e:
                         # If parsed_data["data"] is a string, this won't work - but that's fine
                         pass
@@ -121,9 +119,7 @@ def custom_module_classes():
                         break
                     parsed_data["id"] = parsed_data.get("id", f["s_name"])
                     # Run sample-name cleaning on the data keys
-                    parsed_data["data"] = {
-                        bm.clean_s_name(k, f["root"]): v for k, v in parsed_data.get("data", {}).items()
-                    }
+                    parsed_data["data"] = {bm.clean_s_name(k, f): v for k, v in parsed_data.get("data", {}).items()}
                 elif f_extension == ".png" or f_extension == ".jpeg" or f_extension == ".jpg":
                     image_string = base64.b64encode(f["f"].read()).decode("utf-8")
                     image_format = "png" if f_extension == ".png" else "jpg"
@@ -138,6 +134,8 @@ def custom_module_classes():
                         "section_name": f["s_name"].replace("_", " ").replace("-", " ").replace(".", " "),
                         "data": img_html,
                     }
+                    # If the search pattern 'k' has an associated custom content section config, use it
+                    parsed_data.update(cust_mods.get(k, {}).get("config", {}))
                 elif f_extension == ".html":
                     parsed_data = {"id": f["s_name"], "plot_type": "html", "data": f["f"]}
                     parsed_data.update(_find_html_file_header(f))
@@ -171,11 +169,13 @@ def custom_module_classes():
 
                     # Guess sample name if not given
                     if s_name is None:
-                        s_name = bm.clean_s_name(f["s_name"], f["root"])
+                        s_name = f["s_name"]
 
                     # Guess c_id if no information known
                     if k == "custom_content":
                         c_id = s_name
+                        if not m_config.get("id"):
+                            m_config["id"] = c_id
 
                     # Merge with config from a MultiQC config file if we have it
                     m_config.update(mod_cust_config.get(c_id, {}))
@@ -297,7 +297,7 @@ def custom_module_classes():
 
 
 class MultiqcModule(BaseMultiqcModule):
-    """ Module class, used for each custom content type """
+    """Module class, used for each custom content type"""
 
     def __init__(self, c_id, mod):
 
@@ -318,6 +318,7 @@ class MultiqcModule(BaseMultiqcModule):
             href=mod["config"].get("section_href"),
             info=mod_info,
             extra=mod["config"].get("extra"),
+            # No DOI here.. // doi=
         )
 
         # Don't repeat the Custom Content name in the subtext
@@ -351,21 +352,28 @@ class MultiqcModule(BaseMultiqcModule):
         section_description = mod["config"].get("description", "")
 
         pconfig = mod["config"].get("pconfig", {})
+        if pconfig.get("id") is None:
+            pconfig["id"] = f"{c_id}-plot"
         if pconfig.get("title") is None:
             pconfig["title"] = section_name
 
         plot = None
         content = None
 
+        # Save the data if it's not a html string
+        if not isinstance(mod["data"], str):
+            self.write_data_file(mod["data"], "multiqc_{}".format(pconfig["id"]))
+            pconfig["save_data_file"] = False
+
         # Table
         if mod["config"].get("plot_type") == "table":
             pconfig["sortRows"] = pconfig.get("sortRows", False)
             headers = mod["config"].get("headers")
             plot = table.plot(mod["data"], headers, pconfig)
-            self.write_data_file(mod["data"], "multiqc_{}".format(section_name.lower().replace(" ", "_")))
 
         # Bar plot
         elif mod["config"].get("plot_type") == "bargraph":
+            mod["data"] = {k: v for k, v in sorted(mod["data"].items())}
             plot = bargraph.plot(mod["data"], mod["config"].get("categories"), pconfig)
 
         # Line plot
@@ -407,7 +415,7 @@ class MultiqcModule(BaseMultiqcModule):
         # Don't use exactly the same title / description text as the main module
         if section_name == self.name:
             section_name = None
-        if section_description == self.info:
+        if self.info and section_description.strip(".") == self.info.strip("."):
             section_description = ""
 
         self.add_section(name=section_name, anchor=c_id, description=section_description, plot=plot, content=content)
@@ -435,7 +443,7 @@ def _find_file_header(f):
 
 
 def _find_html_file_header(f):
-    """ Look for a HTML comment config at the start of a custom content HTML file """
+    """Look for a HTML comment config at the start of a custom content HTML file"""
     if f["f"].lstrip().startswith("<!--"):
         match = re.search(r"^\<\!\-\-((?:.|\n|\r)*?)-->", f["f"].lstrip())
         if match:
