@@ -21,6 +21,7 @@ class TsTvByCountMixin:
                 val = float(line.split()[3])  # taking Ts/Tv as value
                 d[key] = val
             self.vcftools_tstv_by_count[f["s_name"]] = d
+            self.add_data_source(f, "TsTv by counts")
 
         # Filter out ignored sample names
         self.vcftools_tstv_by_count = self.ignore_samples(self.vcftools_tstv_by_count)
@@ -28,6 +29,9 @@ class TsTvByCountMixin:
         if len(self.vcftools_tstv_by_count) == 0:
             return 0
 
+        # Write data to file
+        self.write_data_file(self.vcftools_tstv_by_count, "vcftools_tstv_by_count")
+
         pconfig = {
             "id": "vcftools_tstv_by_count",
             "title": "VCFTools: TsTv by Count",
